@@ -94,6 +94,40 @@ static string
 shell_protect(const string & file)
 {
     string safefile = file;
+#ifdef __WIN32__
+    bool need_to_quote = false;
+    for (string::iterator i = safefile.begin(); i != safefile.end(); ++i) {
+	unsigned char ch = *i;
+	if (!isalnum(ch) && ch < 128) {
+	    if (ch == '/') {
+		// Convert Unix path separators to backslashes.  C library
+		// functions understand "/" in paths, but external commands
+		// generally don't, and also may interpret a leading '/' as
+		// introducing a command line option.
+		*i = '\\';
+	    } else if (ch == ' ') {
+		need_to_quote = true;
+	    } else if (ch < 32 || strchr("<>\"|*?", ch)) {
+		// Check for invalid characters in the filename.
+		string m("Invalid character '");
+		m += ch;
+		m += "' in filename \"";
+		m += file;
+		m += '"';
+		throw m;
+	    }
+	}
+    }
+    if (safefile[0] == '-') {
+	// If the filename starts with a '-', protect it from being treated as
+	// an option by prepending ".\".
+	safefile.insert(0, ".\\");
+    }
+    if (need_to_quote) {
+	safefile.insert(0, "\"");
+	safefile += '"';
+    }
+#else
     string::size_type p = 0;
     if (!safefile.empty() && safefile[0] == '-') {
 	// If the filename starts with a '-', protect it from being treated as
@@ -110,6 +144,7 @@ shell_protect(const string & file)
 	}
 	++p;
     }
+#endif
     return safefile;
 }
 
@@ -126,7 +161,7 @@ static bool ensure_tmpdir() {
 	tmpdir.assign(dir_template);
 	tmpdir += '/';
     }
-    delete dir_template;
+    delete [] dir_template;
     return (p != NULL);
 }
 
@@ -208,7 +243,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    text = file_to_string(file);
 	} catch (ReadError) {
-	    cout << "can't read \"" << file << "\" - skipping\n";
+	    cout << "can't read \"" << file << "\" - skipping" << endl;
 	    return;
 	}
 	MyHtmlParser p;
@@ -227,7 +262,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    // indexing is disallowed
 	}
 	if (!p.indexing_allowed) {
-	    cout << "indexing disallowed by meta tag - skipping\n";
+	    cout << "indexing disallowed by meta tag - skipping" << endl;
 	    return;
 	}
 	dump = p.dump;
@@ -256,7 +291,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 		// FIXME: What charset is the file?  Look at contents?
 	    }
 	} catch (ReadError) {
-	    cout << "can't read \"" << file << "\" - skipping\n";
+	    cout << "can't read \"" << file << "\" - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/pdf") {
@@ -265,7 +300,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
 	get_pdf_metainfo(safefile, title, keywords);
@@ -315,7 +350,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    xmlparser.parse_html(stdout_to_string(cmd));
 	    dump = xmlparser.dump;
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
 
@@ -334,7 +369,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/vnd.ms-excel") {
@@ -342,7 +377,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/vnd.ms-powerpoint") {
@@ -350,7 +385,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (startswith(mimetype, "application/vnd.openxmlformats-officedocument.")) {
@@ -364,7 +399,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    args = " ppt/slides/slide*.xml";
 	} else {
 	    // Don't know how to index this type.
-	    cout << "unknown Office 2007 MIME subtype - skipping\n";
+	    cout << "unknown Office 2007 MIME subtype - skipping" << endl;
 	    return;
 	}
 	string safefile = shell_protect(file);
@@ -374,7 +409,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    xmlparser.parse_html(stdout_to_string(cmd));
 	    dump = xmlparser.dump;
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/vnd.wordperfect") {
@@ -385,7 +420,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/vnd.ms-works") {
@@ -394,7 +429,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/x-abiword") {
@@ -406,7 +441,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    dump = xmlparser.dump;
 	    md5_string(text, md5);
 	} catch (ReadError) {
-	    cout << "can't read \"" << file << "\" - skipping\n";
+	    cout << "can't read \"" << file << "\" - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/x-abiword-compressed") {
@@ -417,7 +452,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    xmlparser.parse_html(stdout_to_string(cmd));
 	    dump = xmlparser.dump;
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "text/rtf") {
@@ -430,14 +465,14 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    // produce them.
 	    p.parse_html(stdout_to_string(cmd), "iso-8859-1", true);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	} catch (bool) {
 	    // MyHtmlParser throws a bool to abandon parsing at </body> or when
 	    // indexing is disallowed
 	}
 	if (!p.indexing_allowed) {
-	    cout << "indexing disallowed by meta tag - skipping\n";
+	    cout << "indexing disallowed by meta tag - skipping" << endl;
 	    return;
 	}
 	dump = p.dump;
@@ -452,7 +487,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    dump = stdout_to_string(cmd);
 	    convert_to_utf8(dump, "ISO-8859-1");
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/x-dvi") {
@@ -466,7 +501,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    dump = stdout_to_string(cmd);
 	    convert_to_utf8(dump, "ISO-8859-1");
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "image/vnd.djvu") {
@@ -478,7 +513,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	try {
 	    dump = stdout_to_string(cmd);
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else if (mimetype == "application/vnd.ms-xpsdocument") {
@@ -498,18 +533,18 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    xpsparser.parse_html(dump);
 	    dump = xpsparser.dump;
 	} catch (ReadError) {
-	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    cout << "\"" << cmd << "\" failed - skipping" << endl;
 	    return;
 	}
     } else {
 	// Don't know how to index this type.
-	cout << "unknown MIME type - skipping\n";
+	cout << "unknown MIME type - skipping" << endl;
 	return;
     }
 
     // Compute the MD5 of the file if we haven't already.
     if (md5.empty() && md5_file(file, md5) == 0) {
-	cout << "failed to read file to calculate MD5 checksum - skipping\n";
+	cout << "failed to read file to calculate MD5 checksum - skipping" << endl;
 	return;
     }
 
@@ -588,18 +623,12 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
     if (!skip_duplicates) {
 	// If this document has already been indexed, update the existing
 	// entry.
-	try {
-	    Xapian::docid did = db.replace_document(urlterm, newdocument);
-	    if (did < updated.size()) {
-		updated[did] = true;
-		cout << "updated." << endl;
-	    } else {
-		cout << "added." << endl;
-	    }
-	} catch (...) {
-	    // FIXME: is this ever actually needed?
-	    db.add_document(newdocument);
-	    cout << "added (failed re-seek for duplicate)." << endl;
+	Xapian::docid did = db.replace_document(urlterm, newdocument);
+	if (did < updated.size()) {
+	    updated[did] = true;
+	    cout << "updated." << endl;
+	} else {
+	    cout << "added." << endl;
 	}
     } else {
 	// If this were a duplicate, we'd have skipped it above.
@@ -888,9 +917,9 @@ main(int argc, char **argv)
 	case 's':
 	    try {
 		stemmer = Xapian::Stem(optarg);
-	    } catch (const Xapian::Error &) {
-		cerr << "Unknown stemming language '" << optarg << "'.\n";
-		cerr << "Available language names are: "
+	    } catch (const Xapian::InvalidArgumentError &) {
+		cerr << "Unknown stemming language '" << optarg << "'.\n"
+			"Available language names are: "
 		     << Xapian::Stem::get_available_languages() << endl;
 		return 1;
 	    }
@@ -903,11 +932,11 @@ main(int argc, char **argv)
     }
 
     if (dbpath.empty()) {
-	cerr << PROG_NAME": you must specify a database with --db.\n";
+	cerr << PROG_NAME": you must specify a database with --db." << endl;
 	return 1;
     }
     if (baseurl.empty()) {
-	cerr << PROG_NAME": --url not specified, assuming `/'.\n";
+	cerr << PROG_NAME": --url not specified, assuming `/'." << endl;
     }
     // baseurl mustn't end '/' or you end up with the wrong URL
     // (//thing is different to /thing). We could probably make this
@@ -962,7 +991,7 @@ main(int argc, char **argv)
 	    }
 	}
 	db.flush();
-	// cout << "\n\nNow we have " << db.get_doccount() << " documents.\n";
+	// cout << "\n\nNow we have " << db.get_doccount() << " documents." << endl;
 	exitcode = 0;
     } catch (const Xapian::Error &e) {
 	cout << "Exception: " << e.get_msg() << endl;
