@@ -739,7 +739,7 @@ main(int argc, char **argv)
 	    case 's':
 		try {
 		    stemmer = Xapian::Stem(optarg);
-		} catch (const Xapian::Error &) {
+		} catch (const Xapian::InvalidArgumentError &) {
 		    cerr << "Unknown stemming language '" << optarg << "'.\n";
 		    cerr << "Available language names are: "
 			 << Xapian::Stem::get_available_languages() << endl;
